<?php
	
	/*
	 * This file is responsible for netsuite GET and POST requests
	 * This file uses In8Sync plugin for generating Oauth creds
	 */
	
	define("DEPLOY_API", "1");
	define('CONSUMER_KEY', "9f8a7633325122c952024c16b6a9f86fdc45a37f7d964c5caa3e98deab414d21");
	define('CONSUMER_SECRET', "e249856f635c9721e5f02ed51380127a37974a4ff264cc0c858ade8deb2d23bd");
	
	if (!class_exists('NetsuiteConnection')) {
		
		/**
		 * @property In8_Logger $logger
		 */
		class NetsuiteConnection {
			
			private $consumer, $token, $sig, $url;
			private $url_options = array("deploy" => DEPLOY_API);
			private $account_info = array();
			private $object_id, $debug_level = 0; //api calling
			private $object, $recordType;
			public $logger;
			
			public function __construct($object_id, $object, $account = NULL, $token_id = NULL, $token_secret = NULL, $restlet_url = NULL) {
				
				$this->object_id = $object_id;
				$this->object = $object;
				$this->recordType = str_replace(['salesorder-', 'purchaseorder-', '-search'], ['', '', ''], $object);
				
				$this->account_info = IN8_Settings::getTabSettings('in8_tab_general', 'ns_api_creds');
				
				if (isset($account)) $this->account_info['account'] = $account;
				if (isset($token_id)) $this->account_info['token_id'] = $token_id;
				if (isset($token_secret)) $this->account_info['token_secret'] = $token_secret;
				if (isset($restlet_url)) $this->account_info['restlet_url'] = $restlet_url;
				
				$this->setLogger();
			}
			
			private function setLogger() {
				$this->debug_level = IN8_Settings::getDebuggingLevel();
				if ($this->debug_level != 0) {
					$this->logger = new In8_Logger();
					//not set logger for these records
					if ($this->object == 'customerforms' || $this->object == 'paymentmethod' || $this->object == 'shipitem' || $this->object == 'salesorderforms') {
						return FALSE;
					}
					$log_file = IN8_Settings::getLogFilePath('log', $this->object, $this->object_id);
					
					if ($log_file !== FALSE) {
						$log_file_url = IN8_Settings::getLogFileURL('log', $this->object, $this->object_id);
						$this->logger->lfile($log_file, $log_file_url);
					} else {
						$this->logger->sendErrorEmails("Unable to write logs to plugin or wc-log directory for " . $this->recordType . " resource");
						$this->logger->disableLogger = TRUE;
					}
				}
			}
			
			private function _prepareRequest() {
				//$this->account_info = IN8_Settings::getTabSettings('in8_tab_general', 'ns_api_creds');

//check if API creds are defined
				if (!empty($this->account_info) && !empty($this->account_info['token_id']) && !empty($this->account_info['token_secret'])) {
					require_once( WP_PLUGIN_DIR . '/in8sync/code/libraries/OAuth.php');
					
					$this->url = $this->account_info['restlet_url'] ? $this->account_info['restlet_url'] : "https://rest.na1.netsuite.com/app/site/hosting/restlet.nl";
					$this->url_options['consumerKey'] = CONSUMER_KEY;
					$this->url_options['token'] = $this->account_info['token_id'];
					$this->consumer = new OAuthConsumer(CONSUMER_KEY, CONSUMER_SECRET);
					$this->token = new OAuthToken($this->account_info['token_id'], $this->account_info['token_secret']);
					$this->sig = new OAuthSignatureMethod_HMAC_SHA1(); //Signature
					$this->params = array(
						'oauth_nonce' => generateRandomString(),
						'oauth_timestamp' => idate('U'),
						'oauth_version' => '1.0',
						'oauth_token' => $this->account_info['token_id'],
						'oauth_consumer_key' => CONSUMER_KEY,
						'oauth_signature_method' => $this->sig->get_name()
					);
					return TRUE;
				} else {
					return FALSE;
				}
			}
			
			public function doRequest($data = array(), $url_option = NULL, $return_error = false) {
				
				if ($this->_prepareRequest()) {
					
					$label = 'in8';
					if( $this->account_info['is_spherical'] ){
						$label = 'sph';
					}
					//determine if request is POST or GET
					if (!empty($data)) {
						$method = "POST";
						$this->url_options["script"] = "customscript_".$label."_sync_rl";
					} else {
						$method = "GET";
						$this->url_options["script"] = "customscript_".$label."_search_rl";
						
						if (!empty($url_option)) {
							$this->url_options = array_merge($this->url_options, $url_option);
						}
						
						if (!isset($this->url_options['search']) || $this->url_options["script"] != 'customscript_'.$label.'_inventory_rl') {
							$this->url_options["recordType"] = $this->recordType;
						}
					}
					// On the GET request, include the options
					$this->url .='?' . http_build_query($this->url_options);
//Oauth request
					$req = new OAuthRequest($method, $this->url, $this->params);
					
					$req->set_parameter('oauth_signature', $req->build_signature($this->sig, $this->consumer, $this->token));
					$req->set_parameter('realm', $this->account_info['account']);

//request headers
					$header = [];
					$header[] = $req->to_header() . ',realm="' . $this->account_info['account'] . '"';
//extract host from url
					$parse_url = parse_url($this->url);
					$host = $parse_url['host']; //get host
					$header[] = "Host: " . $host;
					
					$header[] = "Content-Type: application/json";
					
					if ($method == "GET") {
						$json = $this->_getContentsCurl($this->url, $header);
					} else {
						$data_encoded = json_encode($data);
						$header[] = "Content-Length: " . strlen($data_encoded);
						$json = $this->_postContentsCurl($this->url, $header, $data_encoded);
					}
					
					
					if ($this->debug_level > 1) {
						if (!empty($data)) {
							$this->logger->saveRequestJson($data_encoded);
						}
					}
					
					
					$json_decoded = json_decode($json, 1);
					if (!empty($json_decoded)) {
						if (isset($json_decoded['success']) && $json_decoded['success'] == 1) {
							if ($this->debug_level > 0) {
								$this->logger->writeLogToDB(1, '', $this->object, $this->object_id);
								if (file_exists($this->logger->log_file)) {
									@unlink($this->logger->log_file);
								}
							}
							return $json_decoded;
						} else if (isset($json_decoded['error']) && !empty($json_decoded['error'])) {
							$error = "CODE : " . $json_decoded['error']['code'] . ", MESSAGE : " . $json_decoded['error']['message'];
							$error_code = $json_decoded['error']['code'];
						} elseif (isset($json_decoded['errors']) && !empty($json_decoded['errors'])) {
							$error = "CODE : " . $json_decoded['errors'][0]['errortype'] . ", MESSAGE : " . $json_decoded['errors'][0]['message'];
							$error_code = $json_decoded['errors'][0]['errortype'];
						} elseif (!empty($json_decoded)) {
							$error = "UNKNOWN JSON RESPONSE :: " . $json;
							$error_code = "UNKNOWN_RESPONSE";
						}
					} else {
						$error = "API RESPONSE NULL";
						$error_code = "NULL_RESPONSE";
					}
				} else {
					$error = "API credentials not defined";
					$error_code = "API_CREDENTIALS_NOT_DEFINED";
				}
				
				
				if (isset($error) && $this->debug_level > 0) {
					$error_code = isset($error_code) ? $error_code : "";
					$this->logger->processError($error, $error_code, $this->object, $this->object_id);
				}
				
				if( $return_error ){
					$error_message = '';
					if (isset($json_decoded['error']) && !empty($json_decoded['error'])) {
						$error_message = $json_decoded['error']['message'];
					} elseif (isset($json_decoded['errors']) && !empty($json_decoded['errors'])) {
						$error_message = $json_decoded['errors'][0]['message'];
					}
					return ['error' => $error_message];
				}
				return FALSE;
			}
			
			public function processError($error, $error_code = '') {
				if ($this->debug_level > 0) {
					$error = "Level " . $this->debug_level . " :: " . $error;
					$this->logger->lwrite($error); //write to file
					$this->logger->sendErrorNotification($this->object, $error); //send email
					$this->logger->writeLogToDB(0, $error, $this->object, $this->object_id); // write to database
				}
			}
			
			public $message_id = 0;
			public $message_is_error = false;
			public function processMessage($error, $done = false, $is_error = false, $openList = false, $closeList = false) {
				if ($this->debug_level > 0) {
					
					$class = '';
					if( $is_error ){
						$this->message_is_error = true;
						$class = 'error';
					}
					
					$error = "<li class='{$class}'>{$error}</li>";
					if( $openList ){ $error = "<ul class='in8_error_list_class'>{$error}"; }
					if( $closeList ){ $error = "{$error}</ul>"; }
					
					if( $done ){
						if( $this->message_id == 0 ) return false;
						
						global $wpdb;
						$message = $wpdb->get_var("SELECT notes FROM {$wpdb->prefix}in8sync_log WHERE id={$this->message_id}");
						$message = "{$message}{$error}</ul></div>";
						
						// if it's an error message
						if( $this->message_is_error ) {
							$this->logger->lwrite($message); //write to file
							$this->logger->sendErrorNotification($this->object, $message); //send email
						}
						
						$query_array['notes'] = $message;
						$wpdb->update($wpdb->prefix . "in8sync_log", $query_array, ['id'=>$this->message_id]);
						$this->message_id = 0;
						$this->message_is_error = false;
					}else{
						global $wpdb;
						if( $this->message_id == 0 ){
							$status = 0;
							$query_array = ["status" => $status, "woo_object_id" => $this->object_id, "operation" => $this->object];
							$query_array['notes'] = "<div class='in8_error_list_class_main'><ul class='in8_error_list_class'>{$error}";
							$wpdb->insert($wpdb->prefix . "in8sync_log", $query_array);
							$this->message_id = $wpdb->insert_id;
						}else{
							$message = $wpdb->get_var("SELECT notes FROM {$wpdb->prefix}in8sync_log WHERE id={$this->message_id}");
							$query_array['notes'] = "$message ".$error;
							$wpdb->update($wpdb->prefix . "in8sync_log", $query_array, ['id'=> $this->message_id]);
						}
					}
				}
			}

//CURL GET request
			private function _getContentsCurl($url, $headers) {
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, $url);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				$data = curl_exec($ch);
				if (curl_error($ch)) {
					$error = "API REQUEST CURL ERROR : " . curl_error($ch);
					$this->processError($error);
				}
				curl_close($ch);
				return $data;
			}

//CURL POST request
			private function _postContentsCurl($url, $headers, $post_body) {
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, $url);
				curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
				curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
				curl_setopt($ch, CURLOPT_POSTFIELDS, $post_body);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				$data = curl_exec($ch);
				if (curl_error($ch)) {
					$error = "API REQUEST CURL ERROR : " . curl_error($ch);
					$this->processError($error);
				}
				curl_close($ch);
				return $data;
			}
			
		}
		
	}