<?php

    /**
     * CREATE A NEW TAB : 
     * i) Add TAB ID and NAME to the "tabs" static array in "in8_tabs" Class
     * ii) You must create a tab template file under foolder "tabs/templates/". This template file must be named as "in8_tab_" + TAB ID . eg. classes/in8_tab_emails-backend.php
     * iii) Optional : If tab have some option data or processing data need some extra code, then create a file under "classes" folder. This class file must be named as "in8_tab_" + TAB ID + "-backend".eg. classes/in8_tab_emails-backend.php
     */
    class in8_tabs {

        static $tab_prefix = "in8_tab_";
        static $tab_contains_form = [
            'email', 'general'
        ];
        static $tab_contains_functions = [
            'general', 'logs', 'email'
        ];

        function __construct() {
            add_action('in8_netsuite_top_level_tabs_page', [$this, 'addTabContents'], 50);
            //ajax for saving tab settings
            add_action('wp_ajax_in8_tab_save_settings', [$this, 'saveSettings']);

            if (!empty(IN8_Settings::$default_tabs)) {
                foreach (IN8_Settings::$default_tabs as $tab_id => $tab) {
                    if (in_array($tab_id, self::$tab_contains_functions)) {
                        $tab_key = trim(self::$tab_prefix . $tab_id);
                        $tab_class_file = __DIR__ . '/classes/' . $tab_key . "-backend.php";
                        if (file_exists($tab_class_file)) {
                            require_once($tab_class_file);
                        }
                    }
                }
            }
        }

        static function getAllTabs() {
            return apply_filters("in8_admin_tab", IN8_Settings::$default_tabs);
        }

        function addTabContents() {
            
            if (!empty(IN8_Settings::$default_tabs)) {
                
                foreach (IN8_Settings::$default_tabs as $tab_id => $tab) {
                    $tab_key = self::$tab_prefix . $tab_id;

                    $tab_options = IN8_Settings::getTabSettings($tab_key);
                  
                    $tab_template_file = __DIR__ . '/templates/' . $tab_key . ".php";

                    if (file_exists($tab_template_file)) {
                        require_once($tab_template_file);
                    }
                }
            }
        }

        function saveSettings() {
            ini_set('display_startup_errors', 0);
            ini_set('display_errors', 0);
            error_reporting(NULL);

            $tab_key = $_POST['tab_id'];
            $tab_block_id = $_POST['block_id'];

            if (!empty($tab_key)) {
                $tab_id = trim(str_replace(self::$tab_prefix, "", $tab_key));
                if (in_array($tab_id, self::$tab_contains_form)) {
                    $old_value = get_option($tab_key);
                    if ($old_value === FALSE) {
                        $old_value = [];
                    }
                    $posted_data = $_POST[$tab_key];
                    
                    if (!empty($posted_data[$tab_block_id])) {
                        $old_value[$tab_block_id] = $posted_data[$tab_block_id];
                    } else {
                        if (isset($old_value[$tab_block_id])) {
                            unset($old_value[$tab_block_id]);
                        }
                    }
                    
                    update_option($tab_key, $old_value, false);
                    die(json_encode([
                        'type' => 'success',
                    ]));
                }
            }

            die(json_encode([
                'type' => 'faliure',
            ]));
        }
    }

    new in8_tabs();
    