<?php
	
	class ConditionalMapping {
		
		// user_id,email custom fields and woo vendor class magic variables
		public static $woo_user_fields = [
			'user_id',
			'country',
			'state',
			'email',
			'postcode',
			'city',
			'address_1',
			'address_2',
			'shipping_country',
			'shipping_state',
			'shipping_postcode',
			'shipping_city',
			'shipping_address_1',
			'shipping_address_2',
			'is_vat_exempt',
			'calculated_shipping',
			'user_role',
		];
		public static $woo_order_fields = [
			'id',
			'order_type',
			'post_status',
			'customer_note', // The billing address first name.
			'billing_first_name', // The billing address first name.
			'billing_last_name', // The billing address last name.
			'billing_company', // The billing address company.
			'billing_address_1', // The first line of the billing address.
			'billing_address_2', // The second line of the billing address.
			'billing_city', // The city of the billing address.
			'billing_state', // The state of the billing address.
			'billing_postcode', // The postcode of the billing address.
			'billing_country', // The country of the billing address.
			'billing_phone', // The billing phone number.
			'billing_email', // The billing email.
			'shipping_first_name', // The shipping address first name.
			'shipping_last_name', // The shipping address last name.
			'shipping_company', // The shipping address company.
			'shipping_address_1', // The first line of the shipping address.
			'shipping_address_2', // The second line of the shipping address.
			'shipping_city', // The city of the shipping address.
			'shipping_state', // The state of the shipping address.
			'shipping_postcode', // The postcode of the shipping address.
			'shipping_country', // The country of the shipping address.
			'shipping_method_title',
			'cart_discount', // Total amount of discount.
			'cart_discount_tax', // Total amount of discount applied to taxes.
			'customer_user', // User ID who the order belongs to. 0 for guests.
			'order_key', // Random key/password unqique to each order.
			'order_discount', // Stored after tax discounts pre-2.3. Now @deprecated.
			'order_tax', // Stores order tax total.
			'order_shipping_tax', // Stores shipping tax total.
			'order_shipping', // Stores shipping total.
			'order_total', // Stores order total.
			'order_currency', // Stores currency code used for the order.
			'payment_method', // method ID.
			'payment_method_title', // Name of the payment method used.
			'customer_ip_address', // Vendor IP Address.
			'customer_user_agent' // Vendor User agent.
		];
		public static $woo_order_fields_wc3_alias = [
			'post_status' => 'get_status',
		];
		public static $conditional_mapping_ajax_action = 'save_conditional_mapping_form';
		public static $conditional_mapping_ajax_get_template = 'get_conditional_mapping_template';
		
		public function __construct() {
			//ajax for saving plugin admin settings
			add_action( 'wp_ajax_' . self::$conditional_mapping_ajax_action, [
				'ConditionalMapping',
				'saveConditionalMappingForm',
			] );
			
			//ajax for saving plugin admin settings
			add_action( 'wp_ajax_' . self::$conditional_mapping_ajax_get_template, [
				'ConditionalMapping',
				'getConditionalMappingTemplate',
			] );
			//ajax for saving plugin admin settings
			add_action( 'wp_ajax_nopriv_' . self::$conditional_mapping_ajax_get_template, [
				'ConditionalMapping',
				'getConditionalMappingTemplate',
			] );
		}
		
		/**
		 * ajax function for getting conditional mapping template
		 */
		public static function getConditionalMappingTemplate() {
			
			$request          = $_POST;
			$return           = [];
			$return['status'] = 0;
			
			if ( ! empty( $request ) &&
			     ! empty( $request['type'] ) &&
			     ! empty( $request['operator'] ) &&
			     ! empty( $request['index'] ) ) {
				$template = self::getCMTemplate( $request['type'], $request['operator'], $request['index'], [] );
				if ( ! empty( $template ) ) {
					$return['status']   = 1;
					$return['template'] = $template;
				}
			}
			
			echo json_encode( $return );
			die;
		}
		
		public static function saveConditionalMappingForm() {
			
			ini_set( 'display_errors', 1 );
			ini_set( 'display_startup_errors', 1 );
			error_reporting( E_ALL );
			$request = $_REQUEST;
			//if mapping key and mapping values are not empty,
			//this code remove mapping key/value(s) if corresponsing key/value(s) is empty
			//to eliminate empty values
			if ( ! empty( $request ) && ! empty( $request['main_action'] ) ) {
				$option_name = $request['main_action'];
				$request     = array_values( $request['cm'] );
				//validate data
				foreach ( $request as $key => $mapping ) {
					switch ( $mapping['operator'] ) {
						case 1:
							if ( $mapping['wc_field_key'] == '' || $mapping['ns_field_key'] == '' || $mapping['ns_field_value'] == '' ) {
								unset( $request[ $key ] );
							}
							if ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' ) {
								unset( $request[ $key ]['wc_field_value'] );
							}
							break;
						case 2:
							if ( $mapping['ns_field_key'] == '' || $mapping['ns_field_value'] == '' ) {
								unset( $request[ $key ] );
							}
							break;
						case 3:
							if ( $mapping['wc_field_key'] == '' || $mapping['ns_field_key'] == '' ) {
								unset( $request[ $key ] );
							}
							break;
						case 4:
							if ( $mapping['wc_field_check_key'] == '' || $mapping['wc_field_key'] == '' || $mapping['ns_field_key'] == '' ) {
								unset( $request[ $key ] );
							}
							if ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' ) {
								unset( $request[ $key ]['wc_field_value'] );
							}
							break;
						default:
							unset( $request[ $key ] );
							break;
					}
				}
				
				$option_name::saveOptions( $request, 1 ); //parameter two -> 0 : simple options, 1 : conditional mapping
				die( json_encode( [
					'type' => 'success',
				] ) );
			}
			
			die( json_encode( [
				'type'    => 'Error',
				'message' => 'Invalid Request',
			] ) );
		}
		
		static function isJson( $string ) {
			$string = trim( $string );
			if ( ! empty( $string ) ) {
				if ( substr( $string, 0, 1 ) === '{' || substr( $string, 0, 1 ) === '[' ) {
					json_decode( $string );
					if ( json_last_error() == JSON_ERROR_NONE ) {
						return TRUE;
					} else {
						json_decode( json_encode( $string, JSON_UNESCAPED_SLASHES ) );
						
						return ( json_last_error() == JSON_ERROR_NONE );
					}
				}
			}
			
			return FALSE;
		}
		
		public static function getCMTemplate( $type, $operator, $index = 1, $mapping = [] ) {
			if ( $index == 0 ) {
				$index = 1;
			}
			$template = '';
			
			
			switch ( $operator ) {
				case 1:
					if ( empty( $mapping ) ) {
						$mapping['wc_field_key']   = '';
						$mapping['wc_field_value'] = '';
						$mapping['ns_field_key']   = '';
						$mapping['ns_field_value'] = '';
					}
					//JSON
					if ( ! empty( $mapping['ns_field_value'] ) && self::isJson( $mapping['ns_field_value'] ) ) {
						$mapping['ns_field_value'] = str_replace( '"', "", $mapping['ns_field_value'] );
					}
					
					if ( $type == 2 ) {
						$template = '<tr><td><span class="h6">User meta field</span><br/>';
						$template .= '<input class="form-control input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					} elseif ( $type == 1 ) {
						$template = '<tr><td><span class="h6">User field</span><br/>';
						$template .= '<select name="cm[' . $index . '][wc_field_key]">';
						$template .= "<option value='0'>Select a User field</option>";
						foreach ( self::$woo_user_fields as $value ) {
							if ( $value == $mapping['wc_field_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select></div>";
					} elseif ( $type == 3 ) {
						$template = '<tr><td><span class="h6">Order field</span><br/>';
						$template .= '<select name="cm[' . $index . '][wc_field_key]">';
						$template .= "<option value='0'>Select a order field</option>";
						foreach ( self::$woo_order_fields as $value ) {
							if ( $value == $mapping['wc_field_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select></div>";
					} elseif ( $type == 4 ) {
						$template = '<tr><td><span class="h6">Order meta field</span><br/>';
						$template .= '<input class="form-control input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					} elseif ( $type == 5 ) {
						$template = '<tr><td><span class="h6">Form id - Form field</span><br/>';
						$template .= '<input class="col-xs-6 input-sm" type="text" name="cm[' . $index . '][wc_field_value_formid]" value="' . stripslashes($mapping['wc_field_value_formid']) . '"><input class="col-xs-5 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					}
					$not_empty_checked  = '';
					$not_empty_disabled = '';
					
					if ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' ) {
						$not_empty_checked  = "checked";
						$not_empty_disabled = "disabled";
					}
					if ( ! isset( $mapping['wc_field_value'] ) ) {
						$mapping['wc_field_value'] = '';
					}
				
					$template .= '</td><td class="pull-left"><span class="h6">Not empty</span><br/><input type="checkbox" ' . $not_empty_checked . ' name="cm[' . $index . '][not_empty_check]" class="not_empty_check"></td><td class="pull-left"><span class="h6">WC field value</span><br/><input ' . $not_empty_disabled . ' class="form-control input-sm" type="text" name="cm[' . $index . '][wc_field_value]" value="' . stripslashes($mapping['wc_field_value']) . '"></td></tr>'
					             . '<tr><td><span class="h6">NetSuite field</span><br/><input class="form-control input-sm" type="text" name="cm[' . $index . '][ns_field_key]" value="' . stripslashes($mapping['ns_field_key']) . '"></td>'
					             . '<td><span class="h6">NetSuite field value</span><br/><input class="form-control input-sm" type="text" name="cm[' . $index . '][ns_field_value]" value="' . stripslashes($mapping['ns_field_value']) . '"></td></tr>';
					
					break;
				case 2:
					if ( empty( $mapping ) ) {
						$mapping['ns_field_key']   = '';
						$mapping['ns_field_value'] = '';
					}
					
					//JSON
					if ( ! empty( $mapping['ns_field_value'] ) && self::isJson( $mapping['ns_field_value'] ) ) {
						$mapping['ns_field_value'] = str_replace( '"', "", $mapping['ns_field_value'] );
					}
					
					$template = '<tr><td><span class="h6">NetSuite field</span><br/><input class="form-control input-sm" type="text" name="cm[' . $index . '][ns_field_key]" value="' . stripslashes($mapping['ns_field_key']) . '"></td>'
					            . '<td><span class="h6">NetSuite field value</span><br/><input class="form-control input-sm" type="text" name="cm[' . $index . '][ns_field_value]" value="' . stripslashes($mapping['ns_field_value']) . '"></td></tr>';
					break;
				case 3:
					if ( empty( $mapping ) ) {
						$mapping['wc_field_key']          = '';
						$mapping['ns_field_key']          = '';
						$mapping['wc_field_value_prefix'] = '';
					}
					
					$template = '<tr>';
					
					if ( $type == 2 ) {
						$template .= '<td><span class="h6">Prefix to append - User meta field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm" type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '"><input class="col-xs-8 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					} elseif ( $type == 1 ) {
						$template .= '<td><span class="h6">Prefix to append - User field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm " type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '">';
						$template .= '<select class="input-sm" name="cm[' . $index . '][wc_field_key]">';
						$template .= "<option value='0'>Select a User field</option>";
						foreach ( self::$woo_user_fields as $value ) {
							if ( $value == $mapping['wc_field_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select>";
						$template .= '</div>';
					} elseif ( $type == 3 ) {
						$template .= '<td><span class="h6">Prefix to append - Order field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm" type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '">';
						$template .= '<select class="input-sm" name="cm[' . $index . '][wc_field_key]">';
						$template .= "<option value='0'>Select a order field</option>";
						foreach ( self::$woo_order_fields as $value ) {
							if ( $value == $mapping['wc_field_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select>";
						$template .= '</div>';
					} elseif ( $type == 4 ) {
						$template .= '<td><span class="h6">Prefix to append - Order meta field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm" type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '"><input class="col-xs-8 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					} elseif ( $type == 5 ) {
						$template .= '<td><span class="h6">Form id - Form field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-6 input-sm" type="text" name="cm[' . $index . '][wc_field_value_formid]" value="' . stripslashes($mapping['wc_field_value_formid']) . '"><input class="col-xs-6 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					}
					
					$template .= '</td><td><span class="h6">NetSuite field</span><input class="form-control input-sm" style="width:190px;" type="text" name="cm[' . $index . '][ns_field_key]" value="' . stripslashes($mapping['ns_field_key']) . '"></td>';
					$template .= '</tr>';
					break;
				
				case 4:
					if ( empty( $mapping ) ) {
						$mapping['wc_field_key']       = '';
						$mapping['wc_field_check_key'] = '';
						$mapping['ns_field_value']     = '';
					}
					if ( $type == 2 ) {
						$template = '<tr><td><span class="h6">User meta field</span><br/>';
						$template .= '<input class="form-control input-sm" type="text" name="cm[' . $index . '][wc_field_check_key]" value="' . stripslashes($mapping['wc_field_check_key']) . '"></div>';
					} elseif ( $type == 1 ) {
						$template = '<tr><td><span class="h6">User field</span><br/>';
						$template .= '<select name="cm[' . $index . '][wc_field_check_key]">';
						$template .= "<option value='0'>Select a user field</option>";
						foreach ( self::$woo_user_fields as $value ) {
							if ( $value == $mapping['wc_field_check_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select></div>";
					} elseif ( $type == 3 ) {
						$template = '<tr><td><span class="h6">Order field</span><br/>';
						$template .= '<select name="cm[' . $index . '][wc_field_check_key]">';
						$template .= "<option value='0'>Select a order field</option>";
						foreach ( self::$woo_order_fields as $value ) {
							if ( $value == $mapping['wc_field_check_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select></div>";
					} elseif ( $type == 4 ) {
						$template = '<tr><td><span class="h6">Order meta field</span><br/>';
						$template .= '<input class="form-control input-sm" type="text" name="cm[' . $index . '][wc_field_check_key]" value="' . stripslashes($mapping['wc_field_check_key']) . '"></div>';
					} elseif ( $type == 5 ) {
						$template = '<tr><td><span class="h6">Form id - Form field</span><br/>';
						$template .= '<input class="col-xs-5 input-sm" type="text" name="cm[' . $index . '][wc_field_check_formid]" value="' . stripslashes($mapping['wc_field_check_formid']) . '"><input class="col-xs-6 input-sm" type="text" name="cm[' . $index . '][wc_field_check_key]" value="' . stripslashes($mapping['wc_field_check_key']) . '"></div>';
					}
					
					$not_empty_checked  = '';
					$not_empty_disabled = '';
					if ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' ) {
						$not_empty_checked  = "checked";
						$not_empty_disabled = "disabled";
					}
					
					$mapping['wc_field_value'] = isset($mapping['wc_field_value']) ? $mapping['wc_field_value'] : '';
					$template .= '</td><td class="pull-left"><span class="h6">Not empty</span><br/><input type="checkbox" ' . $not_empty_checked . ' name="cm[' . $index . '][not_empty_check]" class="not_empty_check"></td>'
					             . '<td class="pull-left"><span class="h6">WC field value</span><br/><input ' . $not_empty_disabled . ' class="form-control input-sm" type="text" name="cm[' . $index . '][wc_field_value]" value="' . stripslashes($mapping['wc_field_value']) . '"></td></tr>';
					$template .= '<tr>';
					
					if ( $type == 2 ) {
						$template .= '<td><span class="h6">Prefix to append - User meta field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm" type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '"><input class="col-xs-8 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					} elseif ( $type == 1 ) {
						$template .= '<td><span class="h6">Prefix to append - User field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm " type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '">';
						$template .= '<select class="input-sm" name="cm[' . $index . '][wc_field_key]">';
						$template .= "<option value='0'>Select a User field</option>";
						foreach ( self::$woo_user_fields as $value ) {
							if ( $value == $mapping['wc_field_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select>";
						$template .= '</div>';
					} elseif ( $type == 3 ) {
						$template .= '<td><span class="h6">Prefix to append - Order field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm" type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '">';
						$template .= '<select class="input-sm" name="cm[' . $index . '][wc_field_key]">';
						$template .= "<option value='0'>Select a order field</option>";
						foreach ( self::$woo_order_fields as $value ) {
							if ( $value == $mapping['wc_field_key'] ) {
								$template .= "<option value='" . $value . "' selected>" . $value . "</option>";
							} else {
								$template .= "<option value='" . $value . "'>" . $value . "</option>";
							}
						}
						$template .= "</select>";
						$template .= '</div>';
					} elseif ( $type == 4 ) {
						$template .= '<td><span class="h6">Prefix to append - Order meta field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-4 input-sm" type="text" name="cm[' . $index . '][wc_field_value_prefix]" value="' . stripslashes($mapping['wc_field_value_prefix']) . '"><input class="col-xs-8 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					} elseif ( $type == 5 ) {
						$template .= '<td><span class="h6">Form id - Form field</span><br/>';
						$template .= '<div class="row"><input class="col-xs-6 input-sm" type="text" name="cm[' . $index . '][wc_field_value_formid]" value="' . stripslashes($mapping['wc_field_value_formid']) . '"><input class="col-xs-6 input-sm" type="text" name="cm[' . $index . '][wc_field_key]" value="' . stripslashes($mapping['wc_field_key']) . '"></div>';
					}
					
					$template .= '</td><td><span class="h6">NetSuite field</span><input class="form-control input-sm" style="width:190px;" type="text" name="cm[' . $index . '][ns_field_key]" value="' . stripslashes($mapping['ns_field_key']) . '"></td>';
					$template .= '</tr>';
					break;
				default:
					break;
			}
			
			if( !empty($template) ){
				ob_start();
				
				?>
					<tr>
						<td colspan="2">
							<div class="in8-description-field">
								<a href="#" class="trigger">Edit notes</a>
								<div class="row">
									<div class="col-md-4">
										Description
									</div>
									<div class="col-md-8">
										<div class="edit-note">
                                            <textarea name="cm[<?= $index ?>][description]" cols="30" rows="5"><?= stripslashes($mapping['description']) ?></textarea>
                                        </div>
                                        <div class="note">
											<?= stripslashes($mapping['description']) ?>
                                        </div>
									</div>
								</div>
							</div>
						</td>
					</tr>
				<?php
				
				$descriptionField = ob_get_clean();
				$template .= $descriptionField;
			}
			
			return $template;
		}
		
		public static function ConditionalMappingOperators( $mapping, $order = NULL, $user_id = 0, $entry_id = NULL ) {
			$return_value = '';
			
			if ( ( $data = apply_filters( 'in8_mapping_field_value', FALSE, $mapping, $user_id, $order ) ) ) {
				return $data;
			}
			
			switch ( $mapping['operator'] ) {
				case 1:
					if ( $mapping['wc_field_key'] != '' ) {
						$saved_value = self::conditionalMappingGetValue( $mapping, $order, NULL, $user_id, $entry_id );
						
						
						if ( ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' && ! empty( $saved_value ) ) || $saved_value == $mapping['wc_field_value'] || strtolower( $saved_value ) == strtolower( $mapping['wc_field_value'] ) ) {
							$mapping['wc_field_value'] = isset( $mapping['wc_field_value'] ) ? $mapping['wc_field_value'] : '';
							if ( ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' && ! empty( $saved_value ) ) || ( $saved_value == $mapping['wc_field_value'] && $mapping['not_empty_check'] != 'on')  ) {
								
								//json
								if ( ! empty( $mapping['ns_field_value'] ) && self::isJson( $mapping['ns_field_value'] ) ) {
									$mapping['ns_field_value'] = json_decode( stripslashes( $mapping['ns_field_value'] ), 1 );
								}
								$return_value = $mapping['ns_field_value'];
							}
						}
					}
					break;
				case 2:
					if ( $mapping['ns_field_value'] != '' ) {
						//json
						if ( ! empty( $mapping['ns_field_value'] ) && self::isJson( $mapping['ns_field_value'] ) ) {
							$mapping['ns_field_value'] = json_decode( stripslashes( $mapping['ns_field_value'] ), 1 );
						}
						$return_value = $mapping['ns_field_value'];
					}
					break;
				case 3:
					if ( $mapping['wc_field_key'] != '' ) {
						$saved_value = self::conditionalMappingGetValue( $mapping, $order, NULL, $user_id, $entry_id );
						if ( ! empty( $saved_value ) ) {
							$prefix       = isset( $mapping['wc_field_value_prefix'] ) ? $mapping['wc_field_value_prefix'] : '';
							$return_value = $prefix . $saved_value;
						}
					}
					break;
				case 4:
					if ( $mapping['wc_field_key'] != '' && $mapping['wc_field_check_key'] != '' ) {
						
						$check_value = self::conditionalMappingGetValue( $mapping, $order, $mapping['wc_field_check_key'], $user_id, $entry_id );
						
						if ( ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' && ! empty( $check_value ) ) || ( $mapping['not_empty_check'] != 'on' && ($check_value == $mapping['wc_field_value'] || strtolower( $check_value ) == strtolower( $mapping['wc_field_value'] )) ) ) {
							$mapping['wc_field_value'] = isset( $mapping['wc_field_value'] ) ? $mapping['wc_field_value'] : '';
							$saved_value = self::conditionalMappingGetValue( $mapping, $order, NULL, $user_id, $entry_id );
//							if ( ( isset( $mapping['not_empty_check'] ) && $mapping['not_empty_check'] == 'on' && ! empty( $saved_value ) ) || ( $saved_value == $mapping['wc_field_value'] )  ) {
								//json
								if ( ! empty( $mapping['ns_field_value'] ) && self::isJson( $mapping['ns_field_value'] ) ) {
									$mapping['ns_field_value'] = json_decode( stripslashes( $mapping['ns_field_value'] ), 1 );
								}
								$prefix       = isset( $mapping['wc_field_value_prefix'] ) ? $mapping['wc_field_value_prefix'] : '';
								$return_value = $prefix . $saved_value;
//							}
						}
					}
					break;
				
				default:
					break;
			}
			
			return $return_value;
		}
		
		private static function conditionalMappingGetValue( $mapping, $order = NULL, $wc_field_key = NULL, $user_id, $entry_id = NULL ) {
			$return_value = '';
			
			$is_check_field = FALSE;
			if ( is_null( $wc_field_key ) ) {
				$is_check_field = TRUE;
				$wc_field_key   = $mapping['wc_field_key'];
			}
			
			if ( empty( $wc_field_key ) ) {
				return '';
			}
			
			if ( $mapping['type'] == 1 ) {
				if ( $wc_field_key == 'user_id' ) {
					$return_value = $user_id;
				} elseif ( $wc_field_key == 'email' ) {
					$user_info = get_userdata( $user_id );
					if ( isset( $user_info->user_email ) ) {
						$return_value = $user_info->user_email;
					}
				} elseif ( $wc_field_key == 'user_role' ) {
					$user  = get_user_by( 'id', $user_id );
					$roles = apply_filters( 'in8_mapping_user_roles', $user->roles, $mapping );
					$return_value = $roles[0];
				} else {
					$return_value = get_user_meta( $user_id, ( FALSE === strstr( $wc_field_key, 'shipping_' ) ? 'billing_' : '' ) . $wc_field_key, TRUE );
				}
			} else if ( $mapping['type'] == 2 ) {
				$return_value = get_user_meta( $user_id, $wc_field_key, TRUE );
			} elseif ( $mapping['type'] == 5 ) {// gravity form field
				if ( $is_check_field && ! empty( $mapping['wc_field_check_formid'] ) ) {
					$form_id = $mapping['wc_field_check_formid'];
				} else {
					$form_id = $mapping['wc_field_value_formid'];
				}
				$value = self::find_gravity_form_entry( $wc_field_key, $form_id, $user_id, $entry_id, $mapping );
				if ( $value == 'Yes' ) {
					$value = 'T';
				} elseif ( $value == 'No' ) {
					$value = 'F';
				}
				$return_value = $value;
			} elseif ( ! is_null( $order ) ) {
				$wc3_method = 'get_' . $wc_field_key;
				$wc3_alias = isset(self::$woo_order_fields_wc3_alias[$wc_field_key]) ? self::$woo_order_fields_wc3_alias[$wc_field_key] : null;
				
				if ( $mapping['type'] == 3 ) {
					if ( $wc_field_key == 'memo' ) {
						$return_value = $order->get_customer_note();
					} elseif ( $wc_field_key == 'shipping_method_title' ) {
						$return_value = $order->get_shipping_method();
					} elseif ( method_exists( $order, $wc3_method ) ) {
						$return_value = $order->$wc3_method();
					} elseif ( $wc3_alias !== null ) {
						$return_value = $order->$wc3_alias();
					} elseif ( ! empty( $order->$wc_field_key ) ) {
						$return_value = $order->$wc_field_key;
					}
				} elseif ( $mapping['type'] == 4 ) {
					$return_value = get_post_meta( $order->get_id(), $wc_field_key, TRUE );
				}
			}
			
			return $return_value;
		}
		
		public static function gravity_form_selector( $index, $value = '', $classes = '' ) {
			$template = '<select class="' . $classes . ' input-sm" name="cm[' . $index . '][wc_field_value_prefix]">
                                                    <option value="0">-- select form --</option>';
			global $wpdb;
			$forms = $wpdb->get_results( "SELECT * from {$wpdb->prefix}rg_form" );
			foreach ( $forms as $form ) {
				$template .= '<option value="' . $form->id . '" ' . ( $value == $form->id ? 'selected' : '' ) . '>' . $form->title . '</option>';
			}
			$template .= '</select>';
			
			return $template;
		}
		
		public static function find_gravity_form_entry( $field_label, $form_id, $user_id, $entry_id = NULL, $mapping ) {
			global $wpdb;
			if( $entry_id == NULL ) {
				$entry = $wpdb->get_results( "SELECT * from {$wpdb->prefix}rg_lead WHERE form_id={$form_id} AND created_by={$user_id} ORDER BY date_created DESC LIMIT 1" );
			}else{
				//$entry = $wpdb->get_results( "SELECT * from {$wpdb->prefix}rg_lead WHERE id={$entry_id} ORDER BY date_created DESC LIMIT 1" );
				// for the new version of GF
				$gEntry = GFAPI::get_entry($entry_id);
				$entry = [new stdClass()];
				$entry[0]->id = $gEntry['id'];
				$entry[0]->form_id = $gEntry['form_id'];
			}
			if ( empty( $entry ) ) {
				return NULL;
			}
			$entry = $entry[0];
			if ( $form_id != $entry->form_id ) {
				return NULL;
			}
			
			$field_label_value = self::gravity_field_label_value($field_label);
			
			$form        = GFFormsModel::get_form_meta( $form_id );
			$field_found = NULL;
			foreach ( $form['fields'] as $field ) {
				if ( $field->label == $field_label_value ) {
					$field_found = $field;
					break;
				}
			}
			if ( $field_found == NULL ) {
				return NULL;
			}
			
			$lead = GFAPI::get_entry( $entry->id );
			
			$org_value     = RGFormsModel::get_lead_field_value( $lead, $field_found );
			$value         = $org_value;
			$display_value = GFCommon::get_lead_field_display( $field_found, $value, $lead['currency'] );
			
			if ( ! in_array( $field->type, [ 'email', 'fileupload' ] ) ) {
				$value = $display_value;
			}
			if ( is_array( json_decode( $org_value, ARRAY_A ) ) ) {
				$value = $org_value;
			}
			
			$value = self::gravity_field_conv( $value, $org_value, $display_value, $field, $form_id, $field_label, $field_label_value, $mapping );
			$value = apply_filters( 'in8_mapping_gravity_field_value', $value, $field_label_value, $field, $form );
			
			return $value;
			
		}
		public static function gravity_field_label_value($field_label){
		
			if ( ( $test = self::isGFCombineMultiselect( $field_label ) ) != FALSE ) {
				return $test;
			}
			
			return $field_label;
		}
		
		public static function gravity_field_conv( $value, $org_value, $display_value, $field, $form_id, $field_label, $field_label_value, $mapping ) {
			
			
			switch ( $field->type ) {
				case 'fileupload':
					if ( filter_var( $value, FILTER_VALIDATE_URL ) ) {
						$mime  = self::get_mime_type( $value );
						$ext   = self::formatFileTypeForNS( $mime );
						$value = 'file:' . $value . ';' . $ext;
					}
					break;
				case 'checkbox':
					if( count($org_value) == 1 ){
						$tempPop = array_pop( $org_value );
						$value   = !empty( $tempPop ) ? 'T' : 'F';
					}
					break;
				
				
				default:
					break;
			}
			
			if ( is_array( $org_value ) && ( $test = self::isGFCombineMultiselect( $field_label ) ) != FALSE ) {
				$new_value = array_filter($org_value);
				$new_value = implode(',', $new_value);
				return $new_value;
			}
			
			if ( is_array( $org_value ) && ( $test = self::isNSValues( $mapping['ns_field_key'] ) ) != FALSE ) {
				$new_value = array_filter($org_value);
				$new_value = implode(',', $new_value);
				return $new_value;
			}
			
			return $value;
			
		}
		public static function isGFCombineMultiselect($field_label){
			return strpos( $field_label, 'combineMultiselect.' ) === 0 ? str_replace( 'combineMultiselect.', '', $field_label ) : FALSE;
		}
		public static function isNSValues($field_label){
			return strpos($field_label, '.values') > 0;
		}
		
		public static function formatFileTypeForNS( $type ) {
			
			$map = [
				"image/jpeg"                       => "JPGIMAGE",
				"image/x-xbitmap"                  => "BMPIMAGE",
				"text/csv"                         => "CSV",
				"application/vnd.ms-excel"         => "EXCEL",
				"application/​x-​shockwave-​flash" => "FLASH",
				"image/gif"                        => "GIFIMAGE",
				"application/​x-​gzip-​compressed" => "GZIP",
				"text/html"                        => "HTMLDOC",
				"image/ico"                        => "ICON",
				"text/javascript"                  => "JAVASCRIPT",
				"application/json"                 => "JSON",
				"message/rfc822"                   => "MESSAGERFC",
				"audio/mpeg"                       => "MP3",
				"video/mpeg"                       => "MPEGMOVIE",
				"application/vnd.ms-project"       => "MSPROJECT",
				"application/pdf"                  => "PDF",
				"image/pjpeg"                      => "PJPGIMAGE",
				"text/plain"                       => "PLAINTEXT",
				"image/x-png"                      => "PNGIMAGE",
				"image/png"                        => "PNGIMAGE",
				"application/postscript"           => "POSTSCRIPT",
				"application/​vnd.​ms-​powerpoint" => "POWERPOINT",
				"video/quicktime"                  => "QUICKTIME",
				"application/rtf"                  => "RTF",
				"application/sms"                  => "SMS",
				"text/css"                         => "STYLESHEET",
				"image/tiff"                       => "TIFFIMAGE",
				"application/vnd.visio"            => "VISIO",
				"application/msword"               => "WORD",
				"text/xml"                         => "XMLDOC",
				"application/zip"                  => "ZIP",
				"application/octet-stream"         => "WORD",
			];
			
			$format = $map[ $type ];
			
			return $format;
		}
		
		public static function get_mime_type( $filename ) {
			$idx           = explode( '.', $filename );
			$count_explode = count( $idx );
			$idx           = strtolower( $idx[ $count_explode - 1 ] );
			
			$mimet = [
				"323"     => "text/h323",
				"acx"     => "application/internet-property-stream",
				"ai"      => "application/postscript",
				"aif"     => "audio/x-aiff",
				"aifc"    => "audio/x-aiff",
				"aiff"    => "audio/x-aiff",
				"asf"     => "video/x-ms-asf",
				"asr"     => "video/x-ms-asf",
				"asx"     => "video/x-ms-asf",
				"au"      => "audio/basic",
				"avi"     => "video/x-msvideo",
				"axs"     => "application/olescript",
				"bas"     => "text/plain",
				"bcpio"   => "application/x-bcpio",
				"bin"     => "application/octet-stream",
				"bmp"     => "image/bmp",
				"c"       => "text/plain",
				"cat"     => "application/vnd.ms-pkiseccat",
				"cdf"     => "application/x-cdf",
				"cer"     => "application/x-x509-ca-cert",
				"class"   => "application/octet-stream",
				"clp"     => "application/x-msclip",
				"cmx"     => "image/x-cmx",
				"cod"     => "image/cis-cod",
				"cpio"    => "application/x-cpio",
				"crd"     => "application/x-mscardfile",
				"crl"     => "application/pkix-crl",
				"crt"     => "application/x-x509-ca-cert",
				"csh"     => "application/x-csh",
				"css"     => "text/css",
				"dcr"     => "application/x-director",
				"der"     => "application/x-x509-ca-cert",
				"dir"     => "application/x-director",
				"dll"     => "application/x-msdownload",
				"dms"     => "application/octet-stream",
				"doc"     => "application/msword",
				"dot"     => "application/msword",
				"dvi"     => "application/x-dvi",
				"dxr"     => "application/x-director",
				"eps"     => "application/postscript",
				"etx"     => "text/x-setext",
				"evy"     => "application/envoy",
				"exe"     => "application/octet-stream",
				"fif"     => "application/fractals",
				"flr"     => "x-world/x-vrml",
				"gif"     => "image/gif",
				"gtar"    => "application/x-gtar",
				"gz"      => "application/x-gzip",
				"h"       => "text/plain",
				"hdf"     => "application/x-hdf",
				"hlp"     => "application/winhlp",
				"hqx"     => "application/mac-binhex40",
				"hta"     => "application/hta",
				"htc"     => "text/x-component",
				"htm"     => "text/html",
				"html"    => "text/html",
				"htt"     => "text/webviewhtml",
				"ico"     => "image/x-icon",
				"ief"     => "image/ief",
				"iii"     => "application/x-iphone",
				"ins"     => "application/x-internet-signup",
				"isp"     => "application/x-internet-signup",
				"jfif"    => "image/pipeg",
				"jpe"     => "image/jpeg",
				"jpeg"    => "image/jpeg",
				"jpg"     => "image/jpeg",
				"js"      => "application/x-javascript",
				"latex"   => "application/x-latex",
				"lha"     => "application/octet-stream",
				"lsf"     => "video/x-la-asf",
				"lsx"     => "video/x-la-asf",
				"lzh"     => "application/octet-stream",
				"m13"     => "application/x-msmediaview",
				"m14"     => "application/x-msmediaview",
				"m3u"     => "audio/x-mpegurl",
				"man"     => "application/x-troff-man",
				"mdb"     => "application/x-msaccess",
				"me"      => "application/x-troff-me",
				"mht"     => "message/rfc822",
				"mhtml"   => "message/rfc822",
				"mid"     => "audio/mid",
				"mny"     => "application/x-msmoney",
				"mov"     => "video/quicktime",
				"movie"   => "video/x-sgi-movie",
				"mp2"     => "video/mpeg",
				"mp3"     => "audio/mpeg",
				"mpa"     => "video/mpeg",
				"mpe"     => "video/mpeg",
				"mpeg"    => "video/mpeg",
				"mpg"     => "video/mpeg",
				"mpp"     => "application/vnd.ms-project",
				"mpv2"    => "video/mpeg",
				"ms"      => "application/x-troff-ms",
				"mvb"     => "application/x-msmediaview",
				"nws"     => "message/rfc822",
				"oda"     => "application/oda",
				"p10"     => "application/pkcs10",
				"p12"     => "application/x-pkcs12",
				"p7b"     => "application/x-pkcs7-certificates",
				"p7c"     => "application/x-pkcs7-mime",
				"p7m"     => "application/x-pkcs7-mime",
				"p7r"     => "application/x-pkcs7-certreqresp",
				"p7s"     => "application/x-pkcs7-signature",
				"pbm"     => "image/x-portable-bitmap",
				"pdf"     => "application/pdf",
				"pfx"     => "application/x-pkcs12",
				"pgm"     => "image/x-portable-graymap",
				"pko"     => "application/ynd.ms-pkipko",
				"pma"     => "application/x-perfmon",
				"pmc"     => "application/x-perfmon",
				"pml"     => "application/x-perfmon",
				"pmr"     => "application/x-perfmon",
				"pmw"     => "application/x-perfmon",
				"pnm"     => "image/x-portable-anymap",
				"pot"     => "application/vnd.ms-powerpoint",
				"ppm"     => "image/x-portable-pixmap",
				"pps"     => "application/vnd.ms-powerpoint",
				"ppt"     => "application/vnd.ms-powerpoint",
				"prf"     => "application/pics-rules",
				"ps"      => "application/postscript",
				"pub"     => "application/x-mspublisher",
				"qt"      => "video/quicktime",
				"ra"      => "audio/x-pn-realaudio",
				"ram"     => "audio/x-pn-realaudio",
				"ras"     => "image/x-cmu-raster",
				"rgb"     => "image/x-rgb",
				"rmi"     => "audio/mid",
				"roff"    => "application/x-troff",
				"rtf"     => "application/rtf",
				"rtx"     => "text/richtext",
				"scd"     => "application/x-msschedule",
				"sct"     => "text/scriptlet",
				"setpay"  => "application/set-payment-initiation",
				"setreg"  => "application/set-registration-initiation",
				"sh"      => "application/x-sh",
				"shar"    => "application/x-shar",
				"sit"     => "application/x-stuffit",
				"snd"     => "audio/basic",
				"spc"     => "application/x-pkcs7-certificates",
				"spl"     => "application/futuresplash",
				"src"     => "application/x-wais-source",
				"sst"     => "application/vnd.ms-pkicertstore",
				"stl"     => "application/vnd.ms-pkistl",
				"stm"     => "text/html",
				"svg"     => "image/svg+xml",
				"sv4cpio" => "application/x-sv4cpio",
				"sv4crc"  => "application/x-sv4crc",
				"t"       => "application/x-troff",
				"tar"     => "application/x-tar",
				"tcl"     => "application/x-tcl",
				"tex"     => "application/x-tex",
				"texi"    => "application/x-texinfo",
				"texinfo" => "application/x-texinfo",
				"tgz"     => "application/x-compressed",
				"tif"     => "image/tiff",
				"tiff"    => "image/tiff",
				"tr"      => "application/x-troff",
				"trm"     => "application/x-msterminal",
				"tsv"     => "text/tab-separated-values",
				"txt"     => "text/plain",
				"uls"     => "text/iuls",
				"ustar"   => "application/x-ustar",
				"vcf"     => "text/x-vcard",
				"vrml"    => "x-world/x-vrml",
				"wav"     => "audio/x-wav",
				"wcm"     => "application/vnd.ms-works",
				"wdb"     => "application/vnd.ms-works",
				"wks"     => "application/vnd.ms-works",
				"wmf"     => "application/x-msmetafile",
				"wps"     => "application/vnd.ms-works",
				"wri"     => "application/x-mswrite",
				"wrl"     => "x-world/x-vrml",
				"wrz"     => "x-world/x-vrml",
				"xaf"     => "x-world/x-vrml",
				"xbm"     => "image/x-xbitmap",
				"xla"     => "application/vnd.ms-excel",
				"xlc"     => "application/vnd.ms-excel",
				"xlm"     => "application/vnd.ms-excel",
				"xls"     => "application/vnd.ms-excel",
				"xlt"     => "application/vnd.ms-excel",
				"xlw"     => "application/vnd.ms-excel",
				"xof"     => "x-world/x-vrml",
				"xpm"     => "image/x-xpixmap",
				"xwd"     => "image/x-xwindowdump",
				"z"       => "application/x-compress",
				"zip"     => "application/zip",
			];
			
			if ( isset( $mimet[ $idx ] ) ) {
				return $mimet[ $idx ];
			} else {
				return 'application/octet-stream';
			}
		}
		
		
	}
	
	new ConditionalMapping();
    